#ifndef RZB_INTEL_GLOBAL_H
#define RZB_INTEL_GLOBAL_H

#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "rzb_global.h"

/* STRINGS */
#define NULL_ENTRY "NULL"

/* LIMITS */
#define MAX_MAIL_FROM_SIZE    1024
#define MAX_RCPT_TO_SIZE      1024
#define MAX_SUBJECT_SIZE      1024
#define MAX_MSG_FROM_SIZE     1024
#define MAX_MSG_TO_SIZE       1024

/* INTEL DATA TYPES */
typedef enum
{
    NEWDNS,
    NEWWEB,
    NEWMAIL,
    ADD_MAIL_DATA,
    ADD_MAIL_ATTACHMENT
} INTEL_MESSAGE;

typedef enum
{
    HEADER_DATE,
    MSG_ID,
    MIME_VER,
    LANGUAGE,
    MAILER,
    THREAD_IDX,
    BODY,
    XBLOCK,
    MAX_ADD_MAIL_DATA_TYPE
} ADD_MAIL_DATA_TYPES;

/* INTEL TRACKING */
typedef struct _INTEL_TYPE
{
    unsigned msg_type;
} INTEL_TYPE;

/* DNS TRACKING */
typedef struct _DNS_ENTRY
{
    unsigned src_ip;
    unsigned dst_ip;
    unsigned short tx_id;
    unsigned short flags;
    char *queries;
    char *answers;
    char *auth_ns;
    char *add_rec;
} DNS_ENTRY;

typedef struct _DNS_TRACK
{
    unsigned src_ip;
    unsigned dst_ip;
    unsigned short tx_id;
    unsigned short flags;
    unsigned queries_size;
    unsigned answers_size;
    unsigned auth_ns_size;
    unsigned add_rec_size;
} DNS_TRACK;

typedef struct _DNS_TRACK_DATA
{
    char *src_ip;
    char *dst_ip;
    unsigned short tx_id;
    unsigned short flags;
    char *queries;  // NOT REQUIRED
    char *answers;  // NOT REQUIRED
    char *auth_ns;  // NOT REQUIRED
    char *add_rec;  // NOT REQUIRED
} DNS_TRACK_DATA;

/* WEB TRACKING */
typedef struct _WEB_FINAL
{
    unsigned src_ip;
    unsigned dst_ip;
    char *url;
    char *host;
    char *user_agent;
    char *cookie;
    char *header;
} WEB_FINAL;

typedef struct _WEB_ENTRY
{
    struct in_addr src_ip;
    struct in_addr dst_ip;
    char url[1024];
    char host[1024];
    char user_agent[1024];
    char cookie[4096];
    char header[4096];
} WEB_ENTRY;

typedef struct _WEB_TRACK
{
    unsigned src_ip;
    unsigned dst_ip;
    unsigned url_size;
    unsigned host_size;
    unsigned user_agent_size;
    unsigned cookie_size;
    unsigned header_size;
} WEB_TRACK;

/* MAIL TRACKING */

typedef struct _ATTACHMENT_INFO
{
    unsigned      mail_id;
    unsigned char     md5[MD5_SIZE];
    unsigned      size;
    uuid_t            data_type;
    unsigned      name_size;
} ATTACHMENT_INFO;

typedef struct _MAIL_UPDATE
{
    unsigned mail_id;
    unsigned info_type;
    unsigned info_size;
} MAIL_UPDATE;

typedef struct _NEW_MAIL
{
    struct in_addr src_ip;
    struct in_addr dst_ip;
    unsigned   mail_from_size;
    unsigned   rcpt_to_size;
    unsigned   subject_size;
    unsigned   msg_from_size;
    unsigned   msg_to_size;
} NEW_MAIL;

typedef struct _NEW_MAIL_RESPONSE
{
    unsigned mail_id;
} NEW_MAIL_RESPONSE;

/* OTHER DATA STRUCTURES */
typedef struct _NEW_MAIL_BLOBS
{
    char *mail_from;
    char *rcpt_to;
    char *subject;
    char *msg_from;
    char *msg_to;
} NEW_MAIL_BLOBS;

typedef struct _BASE_MAIL_DATA
{
    char *src_ip;
    char *dst_ip;
    char *mail_from;  // NOT REQUIRED
    char *rcpt_to;    // NOT REQUIRED
    char *subject;    // NOT REQUIRED
    char *msg_from;   // NOT REQUIRED
    char *msg_to;     // NOT REQUIRED
} BASE_MAIL_DATA;

#endif /* RZB_INTEL_GLOBAL_H */
