#include <signal.h>

#include "config.h"
#include "rzb_thread.h"

#define maxthreads 100

static volatile unsigned numthreads;

static pthread_mutex_t trackingmutex = PTHREAD_MUTEX_INITIALIZER;

static pthread_attr_t attr;
static pthread_once_t once_control = PTHREAD_ONCE_INIT;

static void initThreading(void)
{
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
}

SO_PUBLIC HRESULT threadme(void *(*fp)(THREADARGS *), THREADARGS *threadargs)
{
    pthread_t thread_id;
    HRESULT retval;
    int rval;

    pthread_once(&once_control, initThreading);

    do
    {
        pthread_mutex_lock(&trackingmutex);
        if (numthreads < maxthreads)
        {
            sigset_t mask;

            sigemptyset(&mask);
            sigaddset(&mask, SIGTERM);
            sigaddset(&mask, SIGQUIT);
            sigaddset(&mask, SIGINT);
            sigaddset(&mask, SIGHUP);
            sigaddset(&mask, SIGUSR1);
            sigaddset(&mask, SIGUSR2);
            sigaddset(&mask, SIGALRM);
            pthread_sigmask(SIG_SETMASK, &mask, NULL);
            rval = pthread_create(&thread_id, &attr, (void *(*)(void *))fp, threadargs);
            sigemptyset(&mask);
            pthread_sigmask(SIG_SETMASK, &mask, NULL);
            if (!rval)
            {
                numthreads++;
                retval = R_SUCCESS;
            }
            else
            {
                free(threadargs);
                printf("pthread_create() failed! (retval=%d) ", rval);
                retval = R_FAIL;
            }
        }
        else
        {
            printf("Maximum thread count of %u reached\n", numthreads);
            retval = R_BUSY;
        }
        pthread_mutex_unlock(&trackingmutex);
        if (retval == R_BUSY)
            sleep(1);
    } while (retval == R_BUSY);
    return retval;
}

SO_PUBLIC void unthreadme(THREADARGS *threadargs)
{
    pthread_mutex_lock(&trackingmutex);
    numthreads--;
    pthread_mutex_unlock(&trackingmutex);

    free(threadargs);
}

SO_PUBLIC unsigned getActiveTheadCount(void)
{
    unsigned num;
    pthread_mutex_lock(&trackingmutex);
    num = numthreads;
    pthread_mutex_unlock(&trackingmutex);

    return num;
}

SO_PUBLIC HRESULT waitForIdle(unsigned to_secs)
{
    for (; to_secs; to_secs--)
    {
        if (!getActiveTheadCount())
            return R_SUCCESS;
        sleep(1);
    }
    return R_FAIL;
}
