#ifndef RZB_WORKSTATION_GLOBAL_H
#define RZB_WORKSTATION_GLOBAL_H

#include <arpa/inet.h>
#include <sys/types.h>
#include "rzb_global.h"

/* Default Sizes */
#define INIT_ROUTE_TABLE_SIZE = 1000;

/* Structures to support initial requests */
typedef enum
{
    ADD_DATA_TYPE,
    ADD_APP_TYPE,
    AUTH_NUGGET,
    SHOW_ROUTING_TABLE,
    SHOW_AUTH_DATA_TYPES,
    SHOW_AUTH_NUGGET_TYPES,
    SHOW_NUG_INFO,
    SHOW_LOGS
} WKST_MESSAGE;

typedef enum
{
    NO_ROUTE_TABLE,
    ROUTE_TABLE_START,
    ROUTE_TABLE_END,
    ROUTE_DATA_ENTRY,
    ROUTE_APP_ENTRY,
    ROUTE_NUGGET_ENTRY
} WKST_RESPONSE;

typedef enum
{
    LOGS_BY_NUMBER,
    LOGS_BY_FUNCTION,
    LOGS_BY_SEVERITY,
    ALL_LOGS
} LOG_TYPES;

typedef struct _LOG_REQUEST_PACKET
{
    unsigned type;
    unsigned data;
} LOG_REQUEST_PACKET;

typedef struct _LOG_RESPONSE_PACKET
{
    unsigned id;
    unsigned timestamp;
    unsigned level;
    unsigned src_size;
    unsigned msg_size;
} LOG_RESPONSE_PACKET;

typedef struct _LOG_BLOCK
{
    unsigned id;        // Log_ID primary key from framework_logs table
    unsigned timestamp; // Seconds since epoch
    unsigned level;     // Syslog level declarations
    unsigned char *src;     // Name of the function submitting the log entry
    unsigned char *msg;     // Text of the log message
    struct _LOG_BLOCK *next; // Pointer to the next log block, NULL is end of list
} LOG_BLOCK;

typedef struct _WKST_REQ_TYPE
{
    unsigned msg_type;
} WKST_REQ_TYPE;

typedef struct _WKST_DATA_RESPONSE
{
    unsigned resp_type;
    unsigned resp_size;
} WKST_DATA_RESPONSE;

typedef struct _ROUTE_HEADER_PACKET
{
    uuid_t datatype;
    char dt_name[MAX_DESCRIPTION_SIZE];
} ROUTE_HEADER_PACKET;

typedef struct _ROUTE_NUGGET_PACKET
{
    unsigned nugid;
    struct in_addr nugaddr;
    unsigned short nugport;
    char name[MAX_DESCRIPTION_SIZE];
} ROUTE_NUGGET_PACKET;

typedef struct _ROUTE_ENTRY_BLOCK
{
    unsigned nugid;
    char nugaddr[INET6_ADDRSTRLEN];
    unsigned short nugport;
    char *name;
    struct _ROUTE_ENTRY_BLOCK *next;
} ROUTE_ENTRY_BLOCK;

typedef struct _APP_ENTRY_BLOCK
{
    char apptype[SIZE_OF_UUID_STRING];
    char *name;
    ROUTE_ENTRY_BLOCK *nug_list;
    struct _APP_ENTRY_BLOCK *next;
} APP_ENTRY_BLOCK;

typedef struct _DATA_ENTRY_BLOCK
{
    char datatype[SIZE_OF_UUID_STRING];
    char *name;
    APP_ENTRY_BLOCK *app_list;
    struct _DATA_ENTRY_BLOCK *next;
} DATA_ENTRY_BLOCK;

#endif /* RZB_WORKSTATION_GLOBAL_H */

