#include <getopt.h>
#include <time.h>
#include <uuid/uuid.h>

#include "config.h"
#include "rzb_conf.h"
#include "rzb_client.h"
#include "rzb_network.h"
#include "rzb_global.h"
#include "rzb_alert_api.h"
#include "rzb_alert_global.h"
#include "rzb_alert_util.h"

#define DEFAULT_DETECT_PORT 2033
#define DEFAULT_DETECT_PORT_S "2033"

#define MESSAGE "Alert Testing"   // 256 character message
#define LONG "LONG"              // Very long forensics information (NOT REQUIRED)
#define DATABLOCK "\x41\x00"        // The actual data that was evaluated (PDF File, etc..)
#define SHORT "Short"            // A short <2k batch of forensics info (NOT REQUIRED)
#define SRCIP "10.4.12.226"
#define DSTIP "216.168.224.2"
#define DATATYPE "005d5464-7a44-4907-af57-4db08a61e13c"
#define NORM "NORM"              // The Normalized version of the data block


int main(int argc, char ** argv)
{
    ALERT data;
    unsigned char tmp_md5[MD5_SIZE];
    uuid_t      data_type;
    uuid_parse (DATATYPE, data_type);
#ifdef JUDGEMENT
    JUDGEMENT verdict;
#endif

    static const struct option long_options[] =
    {
        {"conf",required_argument, NULL, 'c'},
        {"event-id",required_argument, NULL, 'e'},
        {NULL, 0, NULL, 0}
    };

    int optId; /* holds the option index for getopt*/
    const char *configFile = "rzb.conf";
    const char *eventIdStr = NULL;
    int option_index = 0;

    while (1)
    {
        optId = getopt_long(argc, argv, "c:e:", long_options, &option_index);

        if (optId == -1)
            break;

        switch (optId)
        {
            case 'c':
                configFile = optarg;
                break;
            case 'e':
                eventIdStr = optarg;
                break;
            case '?':  /* getopt errors */
                break;
            default:
                break;
        }
    }

    if (!eventIdStr)
    {
        printf("Usage: ./alertNug --event-id=<event_id> [--conf=<configuration file>]\n");
        exit(-1);
    }

    readConfig(&nrtconfig, configFile);

    data.event_id = strtoul(eventIdStr, NULL, 10);
    data.priority = 1;
    snprintf(data.src_ip, sizeof(data.src_ip), "%s", SRCIP);
    snprintf(data.dst_ip, sizeof(data.dst_ip), "%s", DSTIP);
    data.ip_proto = 6;
    data.src_port = 80;
    data.dst_port = 2437;
    data.msg = MESSAGE;
    data.msg_size = sizeof(MESSAGE) - 1;
    data.short_data = SHORT;
    data.sd_size = sizeof(SHORT) - 1;
    md5sum(DATABLOCK, sizeof(DATABLOCK) - 1, tmp_md5);
    data.main_md5 = tmp_md5;
    data.long_data = LONG;
    data.ld_size = sizeof(LONG) - 1;
    uuid_copy(data.dataType, data_type);
    data.data_block = DATABLOCK;
    data.db_size = 2;
    data.norm_block = NORM;
    data.norm_size = sizeof(NORM) - 1;

    printf("Alerting\n");
    sendAlert(&data);

#ifdef JUDGEMENT
    memcpy(verdict.md5, tmp_md5, sizeof(verdict.md5));
    verdict.size = sizeof(DATABLOCK) - 1;
    verdict.flags = KNOWN_BAD;

    printf("Delivering judgement\n");
    deliverJudgement(verdict);
#endif
    printf("Done\n");

    return 0;
}

