#include <arpa/inet.h>

#include "config.h"
#include "output_nugget.h"
#include "rzb_global.h"
#include "rzb_conf.h"
#include "rzb_client.h"
#include "rzb_network.h"
#include "rzb_alert_global.h"
#include "rzb_alert_api.h"

static void printMetaData(BLOCK_META_DATA *metaData)
{
    ALERT_HEADER *alert = (ALERT_HEADER *)metaData->data;
    char ip_str[INET6_ADDRSTRLEN];
    unsigned char *data;
    unsigned size;

    getAlertData(alert->alert_id, ALRT_MESSAGE, &data, &size);

    printf("\n**** NEW ALERT ****\n");
    printf("Alert ID: %u\n", alert->alert_id);
    printf("Message ID: %.*s\n", (int)size, (char *)data);
    printf("Priority: %d\n", alert->priority);
    printf("Source IP: %s\n", inet_ntop(AF_INET, &alert->src_ip, ip_str, sizeof(ip_str)));
    printf("Source Port: %u\n", (unsigned)alert->src_port);
    printf("Destination IP: %s\n", inet_ntop(AF_INET, &alert->dst_ip, ip_str, sizeof(ip_str)));
    printf("Destination Port: %u\n\n", (unsigned)alert->dst_port);

    //sleep(1);

    printf("*** Data blocks ***\n");
    if (alert->flags1 & SHORT_DATA)
    {
        printf("Short data:  ");
        getAlertData(alert->alert_id, SHORT_DATA, &data, &size);
        printf("%.*s\n", (int)size, (char *)data);
    }
    if (alert->flags1 & LONG_DATA)
    {
        printf("Long data:   ");
        getAlertData(alert->alert_id, LONG_DATA, &data, &size);
        printf("%.*s\n", (int)size, (char *)data);
    }
    if (alert->flags1 & FULL_BLOCK)
    {
        printf("Data block:  ");
        getAlertData(alert->alert_id, FULL_BLOCK, &data, &size);
        printf("%.*s\n", (int)size, (char *)data);
    }
    if (alert->flags1 & NORM_BLOCK)
    {
        printf("Norm data:   ");
        getAlertData(alert->alert_id, NORM_BLOCK, &data, &size);
        printf("%.*s\n", (int)size, (char *)data);
    }
    printf("**** END ALERT ****\n");

    free(data);
}

SO_PUBLIC HRESULT initNug(DetectionAPI *detectionObj)
{
    uuid_t list;
    handler_fp fp;

    uuid_copy(list, ALERT_OUTPUT);
    fp = &printMetaData;
    printf("printMetaData is at %p\n", *(void **)&fp);
    detectionObj->registerHandler(&printMetaData, (const uuid_t *)&list, 1, OUTPUT);

    return(R_SUCCESS);
}

