#include <time.h>
#include <getopt.h>

#include "config.h"
#include "rzb_global.h"
#include "rzb_conf.h"
#include "rzb_client.h"
#include "rzb_workstation_api.h"
#include "rzb_workstation_global.h"

int main(int argc, char ** argv)
{
    LOG_BLOCK *block;
    char time_str[1000];
    int num;
    time_t t;
    struct tm *ts;

    static const struct option long_options[] =
    {
        {"conf",required_argument, NULL, 'c'},
        {"log-number",required_argument, NULL, 'n'},
        {NULL, 0, NULL, 0}
    };

    int optId; /* holds the option index for getopt*/
    const char *configFile = "rzb.conf";
    const char *logNumberStr = NULL;
    int option_index = 0;

    while (1)
    {
        optId = getopt_long(argc, argv, "c:n:", long_options, &option_index);

        if (optId == -1)
            break;

        switch (optId)
        {
            case 'c':
                configFile = optarg;
                break;
            case 'n':
                logNumberStr = optarg;
                break;
            case '?':  /* getopt errors */
                break;
            default:
                break;
        }
    }

    readConfig(&nrtconfig, configFile);

    if (logNumberStr)
        num = strtoul(logNumberStr, NULL, 10);
    else
        num = 5;

    printf("Get logs\n");
    if (getLogsByNum(&block, num) != R_SUCCESS)
    {
        printf("Failed out\n");
        return R_FAIL;
    }

    printf("Logs whileloop\n");

    while (block != NULL)
    {
        t = block->timestamp;
        ts = localtime(&t);
        strftime(time_str, sizeof(time_str), "%Y-%m-%d %H:%M:%S %Z", ts);
        printf("Alert ID: %u\n", block->id);
        printf("Timestamp: %s\n", time_str);
        printf("Level: %u\n", block->level);
        printf("Source: %s\n", (char *)block->src);
        printf("Message: %s\n", (char *)block->msg);
        block = block->next;
        printf("\n");
    }

    return R_SUCCESS;
}

